@@ -52,28 +52,49 @@ void cm_AdjustLRU(cm_scache_t *scp)
         cm_data.scacheLRULastp = scp;
 }
 
-/* called with cm_scacheLock write-locked; recycles an existing scp. */
-long cm_RecycleSCache(cm_scache_t *scp, afs_int32 flags)
+/* call with scache write-locked and mutex held */
+void cm_RemoveSCacheFromHashTable(cm_scache_t *scp)
 {
     cm_scache_t **lscpp;
     cm_scache_t *tscp;
     int i;
-
+	
     if (scp->flags & CM_SCACHEFLAG_INHASH) {
 	/* hash it out first */
 	i = CM_SCACHE_HASH(&scp->fid);
 	for (lscpp = &cm_data.hashTablep[i], tscp = cm_data.hashTablep[i];
-	      tscp;
-	      lscpp = &tscp->nextp, tscp = tscp->nextp) {
+	     tscp;
+	     lscpp = &tscp->nextp, tscp = tscp->nextp) {
 	    if (tscp == scp) {
 		*lscpp = scp->nextp;
 		scp->flags &= ~CM_SCACHEFLAG_INHASH;
 		break;
 	    }
 	}
-	osi_assertx(tscp, "afsd: scache hash screwup");
+    }
+}
+
+/* called with cm_scacheLock write-locked; recycles an existing scp. 
+ *
+ * this function ignores all of the locking hierarchy.  
+ */
+long cm_RecycleSCache(cm_scache_t *scp, afs_int32 flags)
+{
+    if (scp->refCount != 0) {
+	return -1;
     }
 
+    if (scp->flags & CM_SCACHEFLAG_SMB_FID) {
+	osi_Log1(afsd_logp,"cm_RecycleSCache CM_SCACHEFLAG_SMB_FID detected scp 0x%p", scp);
+#ifdef DEBUG
+	osi_panic("cm_RecycleSCache CM_SCACHEFLAG_SMB_FID detected",__FILE__,__LINE__);
+#endif
+	return -1;
+    }
+
+    cm_RemoveSCacheFromHashTable(scp);
+
+#if 0
     if (flags & CM_SCACHE_RECYCLEFLAG_DESTROY_BUFFERS) {
 	osi_queueData_t *qdp;
 	cm_buf_t *bufp;
@@ -86,6 +107,8 @@ long cm_RecycleSCache(cm_scache_t *scp, afs_int32 flags)
 		lock_ObtainMutex(&bufp->mx);
 		bufp->cmFlags &= ~CM_BUF_CMSTORING;
 		bufp->flags &= ~CM_BUF_DIRTY;
+		bufp->flags |= CM_BUF_ERROR;
+		bufp->error = VNOVNODE;
 		bufp->dataVersion = -1; /* bad */
 		bufp->dirtyCounter++;
 		if (bufp->flags & CM_BUF_WAITING) {
@@ -104,6 +127,8 @@ long cm_RecycleSCache(cm_scache_t *scp, afs_int32 flags)
 		lock_ObtainMutex(&bufp->mx);
 		bufp->cmFlags &= ~CM_BUF_CMFETCHING;
 		bufp->flags &= ~CM_BUF_DIRTY;
+		bufp->flags |= CM_BUF_ERROR;
+		bufp->error = VNOVNODE;
 		bufp->dataVersion = -1; /* bad */
 		bufp->dirtyCounter++;
 		if (bufp->flags & CM_BUF_WAITING) {
@@ -114,19 +139,23 @@ long cm_RecycleSCache(cm_scache_t *scp, afs_int32 flags)
 		buf_Release(bufp);
 	    }
         }
+	buf_CleanDirtyBuffers(scp); 
     } else {
 	/* look for things that shouldn't still be set */
 	osi_assert(scp->bufWritesp == NULL);
 	osi_assert(scp->bufReadsp == NULL);
     }
+#endif
 
     /* invalidate so next merge works fine;
      * also initialize some flags */
     scp->flags &= ~(CM_SCACHEFLAG_STATD
+		     | CM_SCACHEFLAG_DELETED
 		     | CM_SCACHEFLAG_RO
 		     | CM_SCACHEFLAG_PURERO
 		     | CM_SCACHEFLAG_OVERQUOTA
-		     | CM_SCACHEFLAG_OUTOFSPACE);
+		     | CM_SCACHEFLAG_OUTOFSPACE
+		     | CM_SCACHEFLAG_EACCESS);
     scp->serverModTime = 0;
     scp->dataVersion = 0;
     scp->bulkStatProgress = hzero;
@@ -174,7 +203,6 @@ long cm_RecycleSCache(cm_scache_t *scp, afs_int32 flags)
      * while we hold the global refcount lock.
      */
     cm_FreeAllACLEnts(scp);
-
     return 0;
 }
 
@@ -185,34 +213,77 @@ long cm_RecycleSCache(cm_scache_t *scp, afs_int32 flags)
 cm_scache_t *cm_GetNewSCache(void)
 {
     cm_scache_t *scp;
+    int retry = 0;
 
-  start:
-    if (cm_data.currentSCaches >= cm_data.maxSCaches) {
-        for (scp = cm_data.scacheLRULastp;
-              scp;
-              scp = (cm_scache_t *) osi_QPrev(&scp->q)) {
-            if (scp->refCount == 0) 
-                break;
-        }
-                
-        if (scp) {
-            osi_assert(scp >= cm_data.scacheBaseAddress && scp < (cm_scache_t *)cm_data.hashTablep);
+#if 0
+    /* first pass - look for deleted objects */
+    for ( scp = cm_data.scacheLRULastp;
+	  scp;
+	  scp = (cm_scache_t *) osi_QPrev(&scp->q)) 
+    {
+	osi_assert(scp >= cm_data.scacheBaseAddress && scp < (cm_scache_t *)cm_data.hashTablep);
 
-	    if (!cm_RecycleSCache(scp, 0)) {
-	    
+	if (scp->refCount == 0) {
+	    if (scp->flags & CM_SCACHEFLAG_DELETED) {
+		osi_Log1(afsd_logp, "GetNewSCache attempting to recycle deleted scp 0x%x", scp);
+		if (!cm_RecycleSCache(scp, CM_SCACHE_RECYCLEFLAG_DESTROY_BUFFERS)) {
+
+		    /* we found an entry, so return it */
+		    /* now remove from the LRU queue and put it back at the
+		     * head of the LRU queue.
+		     */
+		    cm_AdjustLRU(scp);
+
+		    /* and we're done */
+		    return scp;
+		} 
+		osi_Log1(afsd_logp, "GetNewSCache recycled failed scp 0x%x", scp);
+	    } else if (!(scp->flags & CM_SCACHEFLAG_INHASH)) {
 		/* we found an entry, so return it */
 		/* now remove from the LRU queue and put it back at the
-		 * head of the LRU queue.
-		 */
+		* head of the LRU queue.
+		*/
 		cm_AdjustLRU(scp);
 
 		/* and we're done */
 		return scp;
-	    } else {
-		/* We don't like this entry, choose another one. */
-		goto start;
 	    }
-        }
+	}	
+    }	
+    osi_Log0(afsd_logp, "GetNewSCache no deleted or recycled entries available for reuse");
+#endif 
+
+    if (cm_data.currentSCaches >= cm_data.maxSCaches) {
+	/* There were no deleted scache objects that we could use.  Try to find
+	 * one that simply hasn't been used in a while.
+	 */
+	    for ( scp = cm_data.scacheLRULastp;
+		  scp;
+		  scp = (cm_scache_t *) osi_QPrev(&scp->q)) 
+	    {
+		/* It is possible for the refCount to be zero and for there still
+		 * to be outstanding dirty buffers.  If there are dirty buffers,
+		 * we must not recycle the scp. */
+		if (scp->refCount == 0 && scp->bufReadsp == NULL && scp->bufWritesp == NULL) {
+		    if (!buf_DirtyBuffersExist(&scp->fid)) {
+			if (!cm_RecycleSCache(scp, 0)) {
+			    /* we found an entry, so return it */
+			    /* now remove from the LRU queue and put it back at the
+			     * head of the LRU queue.
+			     */
+			    cm_AdjustLRU(scp);
+
+			    /* and we're done */
+			    return scp;
+			}
+		    } else {
+			osi_Log1(afsd_logp,"GetNewSCache dirty buffers exist scp 0x%x", scp);
+		    }
+		}	
+	    }
+	    osi_Log1(afsd_logp, "GetNewSCache all scache entries in use (retry = %d)", retry);
+	    
+		return NULL;
     }
         
     /* if we get here, we should allocate a new scache entry.  We either are below
@@ -445,8 +516,10 @@ cm_scache_t *cm_FindSCache(cm_fid_t *fidp)
     cm_scache_t *scp;
 
     hash = CM_SCACHE_HASH(fidp);
-        
-    osi_assert(fidp->cell != 0);
+
+    if (fidp->cell == 0) {
+	return NULL;
+    }
 
     lock_ObtainWrite(&cm_scacheLock);
     for (scp=cm_data.hashTablep[hash]; scp; scp=scp->nextp) {
@@ -461,8 +534,13 @@ cm_scache_t *cm_FindSCache(cm_fid_t *fidp)
     return NULL;
 }
 
+#ifdef DEBUG_REFCOUNT
+long cm_GetSCacheDbg(cm_fid_t *fidp, cm_scache_t **outScpp, cm_user_t *userp,
+                  cm_req_t *reqp, char * file, long line)
+#else
 long cm_GetSCache(cm_fid_t *fidp, cm_scache_t **outScpp, cm_user_t *userp,
                   cm_req_t *reqp)
+#endif
 {
     long hash;
     cm_scache_t *scp;
@@ -482,7 +560,7 @@ long cm_GetSCache(cm_fid_t *fidp, cm_scache_t **outScpp, cm_user_t *userp,
          fidp->volume==cm_data.rootFid.volume &&
          fidp->vnode==0x0 && fidp->unique==0x0)
     {
-        osi_Log0(afsd_logp,"cm_getSCache called with root cell/volume and vnode=0 and unique=0");
+        osi_Log0(afsd_logp,"cm_GetSCache called with root cell/volume and vnode=0 and unique=0");
     }
 
     // yj: check if we have the scp, if so, we don't need
@@ -490,6 +568,10 @@ long cm_GetSCache(cm_fid_t *fidp, cm_scache_t **outScpp, cm_user_t *userp,
     lock_ObtainWrite(&cm_scacheLock);
     for (scp=cm_data.hashTablep[hash]; scp; scp=scp->nextp) {
         if (cm_FidCmp(fidp, &scp->fid) == 0) {
+#ifdef DEBUG_REFCOUNT
+	    afsi_log("%s:%d cm_GetSCache (1) outScpp 0x%p ref %d", file, line, scp, scp->refCount);
+	    osi_Log1(afsd_logp,"cm_GetSCache (1) outScpp 0x%p", scp);
+#endif
             cm_HoldSCacheNoLock(scp);
             *outScpp = scp;
             cm_AdjustLRU(scp);
@@ -514,7 +596,7 @@ long cm_GetSCache(cm_fid_t *fidp, cm_scache_t **outScpp, cm_user_t *userp,
               fidp->volume==AFS_FAKE_ROOT_VOL_ID &&
               fidp->vnode==0x1 && fidp->unique==0x1);
     if (cm_freelanceEnabled && isRoot) {
-        osi_Log0(afsd_logp,"cm_getSCache Freelance and isRoot");
+        osi_Log0(afsd_logp,"cm_GetSCache Freelance and isRoot");
         /* freelance: if we are trying to get the root scp for the first
          * time, we will just put in a place holder entry. 
          */
@@ -522,7 +604,7 @@ long cm_GetSCache(cm_fid_t *fidp, cm_scache_t **outScpp, cm_user_t *userp,
     }
 	  
     if (cm_freelanceEnabled && special) {
-        osi_Log0(afsd_logp,"cm_getSCache Freelance and special");
+        osi_Log0(afsd_logp,"cm_GetSCache Freelance and special");
         if (fidp->vnode > 1 && fidp->vnode <= cm_noLocalMountPoints + 2) {
 	    lock_ObtainMutex(&cm_Freelance_Lock);
             mp =(cm_localMountPoints+fidp->vnode-2)->mountPointStringp;
@@ -531,8 +613,22 @@ long cm_GetSCache(cm_fid_t *fidp, cm_scache_t **outScpp, cm_user_t *userp,
             mp = "";
         }
         scp = cm_GetNewSCache();
-	  
+	if (scp == NULL) {
+	    osi_Log0(afsd_logp,"cm_GetSCache unable to obtain *new* scache entry");
+            lock_ReleaseWrite(&cm_scacheLock);
+	    return CM_ERROR_WOULDBLOCK;
+	}
+
+#if not_too_dangerous
+	/* dropping the cm_scacheLock allows more than one thread
+	 * to obtain the same cm_scache_t from the LRU list.  Since
+	 * the refCount is known to be zero at this point we have to
+	 * assume that no one else is using the one this is returned.
+	 */
+	lock_ReleaseWrite(&cm_scacheLock);
 	lock_ObtainMutex(&scp->mx);
+	lock_ObtainWrite(&cm_scacheLock);
+#endif
         scp->fid = *fidp;
         scp->volp = cm_data.rootSCachep->volp;
         scp->dotdotFid.cell=AFS_FAKE_ROOT_CELL_ID;
@@ -544,6 +640,7 @@ long cm_GetSCache(cm_fid_t *fidp, cm_scache_t **outScpp, cm_user_t *userp,
         cm_data.hashTablep[hash]=scp;
         scp->flags |= CM_SCACHEFLAG_INHASH;
         scp->refCount = 1;
+	osi_Log1(afsd_logp,"cm_GetSCache (freelance) sets refCount to 1 scp 0x%x", scp);
         if (fidp->vnode > 1 && fidp->vnode <= cm_noLocalMountPoints + 2)
             scp->fileType = (cm_localMountPoints+fidp->vnode-2)->fileType;
         else 
@@ -565,9 +662,15 @@ long cm_GetSCache(cm_fid_t *fidp, cm_scache_t **outScpp, cm_user_t *userp,
         scp->group=0;
         scp->dataVersion=cm_data.fakeDirVersion;
         scp->lockDataVersion=-1; /* no lock yet */
+#if not_too_dangerous
 	lock_ReleaseMutex(&scp->mx);
-        *outScpp = scp;
+#endif
+	*outScpp = scp;
         lock_ReleaseWrite(&cm_scacheLock);
+#ifdef DEBUG_REFCOUNT
+	afsi_log("%s:%d cm_GetSCache (2) outScpp 0x%p ref %d", file, line, scp, scp->refCount);
+	osi_Log1(afsd_logp,"cm_GetSCache (2) outScpp 0x%p", scp);
+#endif
         return 0;
     }
     // end of yj code
@@ -591,6 +694,10 @@ long cm_GetSCache(cm_fid_t *fidp, cm_scache_t **outScpp, cm_user_t *userp,
      */
     for (scp=cm_data.hashTablep[hash]; scp; scp=scp->nextp) {
         if (cm_FidCmp(fidp, &scp->fid) == 0) {
+#ifdef DEBUG_REFCOUNT
+	    afsi_log("%s:%d cm_GetSCache (3) outScpp 0x%p ref %d", file, line, scp, scp->refCount);
+	    osi_Log1(afsd_logp,"cm_GetSCache (3) outScpp 0x%p", scp);
+#endif
             cm_HoldSCacheNoLock(scp);
             osi_assert(scp->volp == volp);
             cm_AdjustLRU(scp);
@@ -604,8 +711,25 @@ long cm_GetSCache(cm_fid_t *fidp, cm_scache_t **outScpp, cm_user_t *userp,
         
     /* now, if we don't have the fid, recycle something */
     scp = cm_GetNewSCache();
+    if (scp == NULL) {
+	osi_Log0(afsd_logp,"cm_GetNewSCache unable to obtain *new* scache entry");
+	lock_ReleaseWrite(&cm_scacheLock);
+	return CM_ERROR_WOULDBLOCK;
+    }
+    osi_Log2(afsd_logp,"cm_GetNewSCache returns scp 0x%x flags 0x%x", scp, scp->flags);
+
     osi_assert(!(scp->flags & CM_SCACHEFLAG_INHASH));
+
+#if not_too_dangerous
+    /* dropping the cm_scacheLock allows more than one thread
+     * to obtain the same cm_scache_t from the LRU list.  Since
+     * the refCount is known to be zero at this point we have to
+     * assume that no one else is using the one this is returned.
+     */
+    lock_ReleaseWrite(&cm_scacheLock);
     lock_ObtainMutex(&scp->mx);
+    lock_ObtainWrite(&cm_scacheLock);
+#endif
     scp->fid = *fidp;
     scp->volp = volp;	/* a held reference */
 
@@ -627,7 +751,10 @@ long cm_GetSCache(cm_fid_t *fidp, cm_scache_t **outScpp, cm_user_t *userp,
     cm_data.hashTablep[hash] = scp;
     scp->flags |= CM_SCACHEFLAG_INHASH;
     scp->refCount = 1;
+    osi_Log1(afsd_logp,"cm_GetSCache sets refCount to 1 scp 0x%x", scp);
+#if not_too_dangerous
     lock_ReleaseMutex(&scp->mx);
+#endif
 
     /* XXX - The following fields in the cm_scache are 
      * uninitialized:
@@ -639,6 +766,10 @@ long cm_GetSCache(cm_fid_t *fidp, cm_scache_t **outScpp, cm_user_t *userp,
         
     /* now we have a held scache entry; just return it */
     *outScpp = scp;
+#ifdef DEBUG_REFCOUNT
+    afsi_log("%s:%d cm_GetSCache (4) outScpp 0x%p ref %d", file, line, scp, scp->refCount);
+    osi_Log1(afsd_logp,"cm_GetSCache (4) outScpp 0x%p", scp);
+#endif
     return 0;
 }
 
@@ -651,22 +782,22 @@ cm_scache_t * cm_FindSCacheParent(cm_scache_t * scp)
     cm_fid_t    parent_fid;
     cm_scache_t * pscp = NULL;
 
-    lock_ObtainWrite(&cm_scacheLock);
+    lock_ObtainRead(&cm_scacheLock);
     parent_fid = scp->fid;
     parent_fid.vnode = scp->parentVnode;
     parent_fid.unique = scp->parentUnique;
 
     if (cm_FidCmp(&scp->fid, &parent_fid)) {
-	for (i=0; i<cm_data.hashTableSize; i++) {
-	    for (pscp = cm_data.hashTablep[i]; pscp; pscp = pscp->nextp) {
-		if (!cm_FidCmp(&pscp->fid, &parent_fid)) {
-		    cm_HoldSCacheNoLock(pscp);
-		    break;
-		}
+	i = CM_SCACHE_HASH(&parent_fid);
+	for (pscp = cm_data.hashTablep[i]; pscp; pscp = pscp->nextp) {
+	    if (!cm_FidCmp(&pscp->fid, &parent_fid)) {
+		cm_HoldSCacheNoLock(pscp);
+		break;
 	    }
 	}
     }
-    lock_ReleaseWrite(&cm_scacheLock);
+
+    lock_ReleaseRead(&cm_scacheLock);
 
     return pscp;
 }
@@ -734,22 +865,16 @@ long cm_SyncOp(cm_scache_t *scp, cm_buf_t *bufp, cm_user_t *userp, cm_req_t *req
     cm_buf_t *tbufp;
     afs_uint32 outRights;
     int bufLocked;
+    afs_uint32 sleep_scp_flags = 0;
+    afs_uint32 sleep_buf_cmflags = 0;
+    afs_uint32 sleep_scp_bufs = 0;
 
     /* lookup this first */
     bufLocked = flags & CM_SCACHESYNC_BUFLOCKED;
 
-    /* some minor assertions */
-    if (flags & (CM_SCACHESYNC_STOREDATA | CM_SCACHESYNC_FETCHDATA
-                  | CM_SCACHESYNC_READ | CM_SCACHESYNC_WRITE
-                  | CM_SCACHESYNC_SETSIZE)) {
-        if (bufp) {
-            osi_assert(bufp->refCount > 0);
-            /*
-               osi_assert(cm_FidCmp(&bufp->fid, &scp->fid) == 0);
-             */
-        }
-    }
-    else osi_assert(bufp == NULL);
+	if (bufp)
+		osi_assert(bufp->refCount > 0);
+
 
     /* Do the access check.  Now we don't really do the access check
      * atomically, since the caller doesn't expect the parent dir to be
@@ -900,7 +1025,7 @@ long cm_SyncOp(cm_scache_t *scp, cm_buf_t *bufp, cm_user_t *userp, cm_req_t *req
         // yj: modified this so that callback only checked if we're
         // not checking something on /afs
         /* fix the conditional to match the one in cm_HaveCallback */
-        if (  (flags & CM_SCACHESYNC_NEEDCALLBACK)
+        if ((flags & CM_SCACHESYNC_NEEDCALLBACK)
 #ifdef AFS_FREELANCE_CLIENT
              && (!cm_freelanceEnabled || 
                   !(scp->fid.vnode==0x1 && scp->fid.unique==0x1) ||
@@ -909,12 +1034,12 @@ long cm_SyncOp(cm_scache_t *scp, cm_buf_t *bufp, cm_user_t *userp, cm_req_t *req
                   cm_fakeDirCallback < 2)
 #endif /* AFS_FREELANCE_CLIENT */
              ) {
-            if (!cm_HaveCallback(scp)) {
+            if ((flags & CM_SCACHESYNC_FORCECB) || !cm_HaveCallback(scp)) {
                 osi_Log1(afsd_logp, "CM SyncOp getting callback on scp 0x%p",
                           scp);
                 if (bufLocked) 
 		    lock_ReleaseMutex(&bufp->mx);
-                code = cm_GetCallback(scp, userp, reqp, 0);
+                code = cm_GetCallback(scp, userp, reqp, (flags & CM_SCACHESYNC_FORCECB)?1:0);
                 if (bufLocked) {
                     lock_ReleaseMutex(&scp->mx);
                     lock_ObtainMutex(&bufp->mx);
@@ -922,6 +1047,7 @@ long cm_SyncOp(cm_scache_t *scp, cm_buf_t *bufp, cm_user_t *userp, cm_req_t *req
                 }
                 if (code) 
                     return code;
+		flags &= ~CM_SCACHESYNC_FORCECB;	/* only force once */
                 continue;
             }
         }
@@ -962,6 +1088,10 @@ long cm_SyncOp(cm_scache_t *scp, cm_buf_t *bufp, cm_user_t *userp, cm_req_t *req
         if (flags & CM_SCACHESYNC_NOWAIT) 
             return CM_ERROR_WOULDBLOCK;
 
+	sleep_scp_flags = scp->flags;		/* so we know why we slept */
+	sleep_buf_cmflags = bufp ? bufp->cmFlags : 0;
+	sleep_scp_bufs = (scp->bufReadsp ? 1 : 0) | (scp->bufWritesp ? 2 : 0);
+
         /* wait here, then try again */
         osi_Log1(afsd_logp, "CM SyncOp sleeping scp 0x%p", scp);
         if ( scp->flags & CM_SCACHEFLAG_WAITING ) {
@@ -1059,6 +1189,8 @@ void cm_SyncOpDone(cm_scache_t *scp, cm_buf_t *bufp, afs_uint32 flags)
     osi_queueData_t *qdp;
     cm_buf_t *tbufp;
 
+    lock_AssertMutex(&scp->mx);
+
     /* now, update the recorded state for RPC-type calls */
     if (flags & CM_SCACHESYNC_FETCHSTATUS)
         scp->flags &= ~CM_SCACHEFLAG_FETCHING;
@@ -1080,19 +1212,24 @@ void cm_SyncOpDone(cm_scache_t *scp, cm_buf_t *bufp, afs_uint32 flags)
         /* ensure that the buffer isn't already in the I/O list */
         for(qdp = scp->bufReadsp; qdp; qdp = (osi_queueData_t *) osi_QNext(&qdp->q)) {
             tbufp = osi_GetQData(qdp);
-            if (tbufp == bufp) break;
+            if (tbufp == bufp) 
+		break;
         }
-        osi_assert(qdp != NULL);
-        osi_assert(osi_GetQData(qdp) == bufp);
-        osi_QRemove((osi_queue_t **) &scp->bufReadsp, &qdp->q);
-        osi_QDFree(qdp);
+	if (qdp) {
+	    osi_QRemove((osi_queue_t **) &scp->bufReadsp, &qdp->q);
+	    osi_QDFree(qdp);
+	}
         if (bufp) {
+	    int release = 0;
+	    if (bufp->cmFlags & CM_BUF_CMFETCHING)
+		release = 1;
             bufp->cmFlags &= ~(CM_BUF_CMFETCHING | CM_BUF_CMFULLYFETCHED);
             if (bufp->flags & CM_BUF_WAITING) {
                 osi_Log2(afsd_logp, "CM SyncOpDone Waking [scp 0x%p] bufp 0x%p", scp, bufp);
                 osi_Wakeup((LONG_PTR) &bufp);
             }
-            buf_Release(bufp);
+	    if (release)
+		buf_Release(bufp);
         }
     }
 
@@ -1101,19 +1238,24 @@ void cm_SyncOpDone(cm_scache_t *scp, cm_buf_t *bufp, afs_uint32 flags)
         /* ensure that the buffer isn't already in the I/O list */
         for(qdp = scp->bufWritesp; qdp; qdp = (osi_queueData_t *) osi_QNext(&qdp->q)) {
             tbufp = osi_GetQData(qdp);
-            if (tbufp == bufp) break;
+            if (tbufp == bufp) 
+		break;
         }
-        osi_assert(qdp != NULL);
-        osi_assert(osi_GetQData(qdp) == bufp);
-        osi_QRemove((osi_queue_t **) &scp->bufWritesp, &qdp->q);
-        osi_QDFree(qdp);
+	if (qdp) {
+	    osi_QRemove((osi_queue_t **) &scp->bufWritesp, &qdp->q);
+	    osi_QDFree(qdp);
+	}
         if (bufp) {
+	    int release = 0;
+	    if (bufp->cmFlags & CM_BUF_CMSTORING)
+		release = 1;
             bufp->cmFlags &= ~CM_BUF_CMSTORING;
             if (bufp->flags & CM_BUF_WAITING) {
                 osi_Log2(afsd_logp, "CM SyncOpDone Waking [scp 0x%p] bufp 0x%p", scp, bufp);
                 osi_Wakeup((LONG_PTR) &bufp);
             }
-            buf_Release(bufp);
+	    if (release)
+		buf_Release(bufp);
         }
     }
 
@@ -1303,34 +1445,68 @@ void cm_AFSFidFromFid(AFSFid *afsFidp, cm_fid_t *fidp)
     afsFidp->Unique = fidp->unique;
 }       
 
+#ifdef DEBUG_REFCOUNT
+void cm_HoldSCacheNoLockDbg(cm_scache_t *scp, char * file, long line)
+#else
 void cm_HoldSCacheNoLock(cm_scache_t *scp)
+#endif
 {
     osi_assert(scp != 0);
-    osi_assert(scp->refCount >= 0);
     scp->refCount++;
+#ifdef DEBUG_REFCOUNT
+    osi_Log2(afsd_logp,"cm_HoldSCacheNoLock scp 0x%p ref %d",scp, scp->refCount);
+    afsi_log("%s:%d cm_HoldSCacheNoLock scp 0x%p, ref %d", file, line, scp, scp->refCount);
+#endif
 }
 
+#ifdef DEBUG_REFCOUNT
+void cm_HoldSCacheDbg(cm_scache_t *scp, char * file, long line)
+#else
 void cm_HoldSCache(cm_scache_t *scp)
+#endif
 {
     osi_assert(scp != 0);
     lock_ObtainWrite(&cm_scacheLock);
-    osi_assert(scp->refCount >= 0);
     scp->refCount++;
+#ifdef DEBUG_REFCOUNT
+    osi_Log2(afsd_logp,"cm_HoldSCache scp 0x%p ref %d",scp, scp->refCount);
+    afsi_log("%s:%d cm_HoldSCache scp 0x%p ref %d", file, line, scp, scp->refCount);
+#endif
     lock_ReleaseWrite(&cm_scacheLock);
 }
 
+#ifdef DEBUG_REFCOUNT
+void cm_ReleaseSCacheNoLockDbg(cm_scache_t *scp, char * file, long line)
+#else
 void cm_ReleaseSCacheNoLock(cm_scache_t *scp)
+#endif
 {
-    osi_assert(scp != 0);
+    osi_assert(scp != NULL);
+    if (scp->refCount == 0)
+	osi_Log1(afsd_logp,"cm_ReleaseSCacheNoLock about to panic scp 0x%x",scp);
     osi_assert(scp->refCount-- >= 0);
+#ifdef DEBUG_REFCOUNT
+    osi_Log2(afsd_logp,"cm_ReleaseSCacheNoLock scp 0x%p ref %d",scp,scp->refCount);
+    afsi_log("%s:%d cm_ReleaseSCacheNoLock scp 0x%p ref %d", file, line, scp, scp->refCount);
+#endif
 }
 
+#ifdef DEBUG_REFCOUNT
+void cm_ReleaseSCacheDbg(cm_scache_t *scp, char * file, long line)
+#else
 void cm_ReleaseSCache(cm_scache_t *scp)
+#endif
 {
-    osi_assert(scp != 0);
+    osi_assert(scp != NULL);
     lock_ObtainWrite(&cm_scacheLock);
+    if (scp->refCount == 0)
+	osi_Log1(afsd_logp,"cm_ReleaseSCache about to panic scp 0x%x",scp);
     osi_assert(scp->refCount != 0);
     scp->refCount--;
+#ifdef DEBUG_REFCOUNT
+    osi_Log2(afsd_logp,"cm_ReleaseSCache scp 0x%p ref %d",scp,scp->refCount);
+    afsi_log("%s:%d cm_ReleaseSCache scp 0x%p ref %d", file, line, scp, scp->refCount);
+#endif
     lock_ReleaseWrite(&cm_scacheLock);
 }
 
