@@ -383,7 +383,7 @@ typedef struct smb_fid {
     afs_uint32	delete;			/* ok to del: locked by smb_rctLock */
 } smb_fid_t;
 
-#define SMB_FID_OPENREAD		1	/* open for reading */
+#define SMB_FID_OPENREAD_LISTDIR	1	/* open for reading / listing directory */
 #define SMB_FID_OPENWRITE		2	/* open for writing */
 #define SMB_FID_CREATED                 4       /* a new file */
 #define SMB_FID_IOCTL			8	/* a file descriptor for the
@@ -398,6 +398,8 @@ typedef struct smb_fid {
 #define SMB_FID_MTIMESETDONE		0x80	/* have set modtime via Tr2 */
 #define SMB_FID_LOOKSLIKECOPY	(SMB_FID_LENGTHSETDONE | SMB_FID_MTIMESETDONE)
 #define SMB_FID_NTOPEN			0x100	/* have dscp and pathp */
+#define SMB_FID_SEQUENTIAL		0x200
+#define SMB_FID_RANDOM			0x400
 
 #define SMB_FID_SHARE_READ              0x1000
 #define SMB_FID_SHARE_WRITE             0x2000
@@ -607,7 +609,9 @@ extern unsigned char *smb_GetSMBData(smb_packet_t *smbp, int *nbytesp);
 
 extern void smb_SetSMBDataLength(smb_packet_t *smbp, unsigned int dsize);
 
-extern unsigned int smb_GetSMBParm(smb_packet_t *smbp, int parm);
+extern unsigned short smb_GetSMBParm(smb_packet_t *smbp, int parm);
+
+extern unsigned char smb_GetSMBParmByte(smb_packet_t *smbp, int parm);
 
 extern unsigned int smb_GetSMBParmLong(smb_packet_t *smbp, int parm);
 
@@ -744,6 +748,13 @@ extern DWORD smb_ServerExceptionFilter(void);
 extern void smb_UpdateServerPriority(void);
 extern void smb_SetRequestStartTime(void);
 extern void smb_ResetServerPriority(void);
+extern void smb_RestartListeners(void);
+extern void smb_StopListeners(void);
+extern void smb_StopListener(NCB *ncbp, int lana);
+
+#define SMB_LISTENER_UNINITIALIZED -1
+#define SMB_LISTENER_STOPPED 0
+#define SMB_LISTENER_STARTED 1
 
 /* include other include files */
 #include "smb3.h"
