@@ -75,7 +75,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/afs_cbqueue.c,v 1.9.2.1 2005/05/30 04:05:40 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/afs_cbqueue.c,v 1.13 2006/03/02 06:42:47 shadow Exp $");
 
 #include "afs/sysincludes.h"	/*Standard vendor system headers */
 #include "afsincludes.h"	/*AFS-based standard headers */
@@ -141,7 +141,6 @@ afs_DequeueCallback(struct vcache *avc)
     debugvc = avc;
     if (avc->callsort.prev) {
 	QRemove(&(avc->callsort));
-	avc->callsort.prev = avc->callsort.next = NULL;
     } else;			/* must have got dequeued in a race */
 
     return;
@@ -220,10 +219,10 @@ afs_CheckCallbacks(unsigned int secs)
 			    /* What about locking xvcache or vrefcount++ or
 			     * write locking tvc? */
 			    QRemove(tq);
-			    tq->prev = tq->next = NULL;
 			    tvc->states &= ~(CStatd | CMValid | CUnique);
-			    if ((tvc->fid.Fid.Vnode & 1)
-				|| (vType(tvc) == VDIR))
+                            if (!(tvc->states & (CVInit|CVFlushed)) &&
+                                (tvc->fid.Fid.Vnode & 1 ||
+                                 (vType(tvc) == VDIR)))
 				osi_dnlc_purgedp(tvc);
 			    tvc->dchint = NULL;	/*invalidate em */
 			    afs_ResetVolumeInfo(tvp);
@@ -237,9 +236,9 @@ afs_CheckCallbacks(unsigned int secs)
 		 * What about locking xvcache or vrefcount++ or write locking tvc?
 		 */
 		QRemove(tq);
-		tq->prev = tq->next = NULL;
 		tvc->states &= ~(CStatd | CMValid | CUnique);
-		if ((tvc->fid.Fid.Vnode & 1) || (vType(tvc) == VDIR))
+                if (!(tvc->states & (CVInit|CVFlushed)) &&
+                    (tvc->fid.Fid.Vnode & 1 || (vType(tvc) == VDIR)))
 		    osi_dnlc_purgedp(tvc);
 	    }
 	}
@@ -309,9 +308,11 @@ afs_FlushCBs(void)
 	    tvc->callback = 0;
 	    tvc->dchint = NULL;	/* invalidate hints */
 	    tvc->states &= ~(CStatd);
-	    if ((tvc->fid.Fid.Vnode & 1) || (vType(tvc) == VDIR))
+	    if (QPrev(&(tvc->callsort)))
+		QRemove(&(tvc->callsort));
+	    if (!(tvc->states & (CVInit|CVFlushed)) &&
+                ((tvc->fid.Fid.Vnode & 1) || (vType(tvc) == VDIR)))
 		osi_dnlc_purgedp(tvc);
-	    tvc->callsort.prev = tvc->callsort.next = NULL;
 	}
 
     afs_InitCBQueue(0);
@@ -338,7 +339,8 @@ afs_FlushServerCBs(struct server *srvp)
 		tvc->callback = 0;
 		tvc->dchint = NULL;	/* invalidate hints */
 		tvc->states &= ~(CStatd);
-		if ((tvc->fid.Fid.Vnode & 1) || (vType(tvc) == VDIR)) {
+		if (!(tvc->states & (CVInit|CVFlushed)) &&
+                    ((tvc->fid.Fid.Vnode & 1) || (vType(tvc) == VDIR))) {
 		    osi_dnlc_purgedp(tvc);
 		}
 		afs_DequeueCallback(tvc);
