@@ -19,7 +19,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/afs_volume.c,v 1.31 2006/02/18 04:08:05 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/afs_volume.c,v 1.31.2.4 2006/12/19 02:34:05 shadow Exp $");
 
 #include "afs/stds.h"
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
@@ -40,6 +40,7 @@ RCSID
 
 #include "afsincludes.h"	/* Afs-based standard headers */
 #include "afs/afs_stats.h"	/* afs statistics */
+#include "afs/afs_dynroot.h"
 
 #if	defined(AFS_SUN56_ENV)
 #include <inet/led.h>
@@ -432,7 +433,7 @@ afs_GetVolume(struct VenusFid *afid, struct vrequest *areq,
 
     tv = afs_FindVolume(afid, locktype);
     if (!tv) {
-	if (afs_IsDynrootFid(afid)) {
+	if (afs_IsDynrootAnyFid(afid)) {
 	    tv = afs_NewDynrootVolume(afid);
 	} else {
 	    bp = afs_cv2string(&tbuf[CVBS], afid->Fid.Volume);
@@ -523,6 +524,11 @@ afs_SetupVolume(afs_int32 volid, char *aname, void *ve, struct cell *tcell,
 	} else {
 	    tv->vtix = -1;
 	    tv->rootVnode = tv->rootUnique = 0;
+            afs_GetDynrootMountFid(&tv->dotdot);
+            afs_GetDynrootMountFid(&tv->mtpoint);
+            tv->mtpoint.Fid.Vnode =
+              VNUM_FROM_TYPEID(VN_TYPE_MOUNT, tcell->cellIndex << 2);
+            tv->mtpoint.Fid.Unique = volid;
 	}
     }
     tv->refCount++;
@@ -682,6 +688,8 @@ afs_NewVolumeByName(char *aname, afs_int32 acell, int agood,
     } while (afs_Analyze(tconn, code, NULL, &treq, -1,	/* no op code for this */
 			 SHARED_LOCK, tcell));
 
+    /* RT 48959 - unclear if this should really go */
+#if 0
     if (code) {
 	/* If the client has yet to contact this cell and contact failed due
 	 * to network errors, mark the VLDB servers as back up.
@@ -704,6 +712,7 @@ afs_NewVolumeByName(char *aname, afs_int32 acell, int agood,
 	afs_PutCell(tcell, READ_LOCK);
 	return NULL;
     }
+#endif
     /*
      * Check to see if this cell has not yet referenced a volume.  If
      * it hasn't, it's just about to change its status, and we need to mark
