@@ -17,7 +17,7 @@
 #endif
 
 RCSID
-    ("$Header: /cvs/openafs/src/auth/ktc.c,v 1.15.2.1 2004/08/25 07:09:36 shadow Exp $");
+    ("$Header: /cvs/openafs/src/auth/ktc.c,v 1.16.8.3 2006/07/19 21:17:17 shadow Exp $");
 
 #if defined(UKERNEL)
 #include "afs/sysincludes.h"
@@ -71,7 +71,7 @@ RCSID
 
 /* For malloc() */
 #include <stdlib.h>
-
+#include "ktc.h"
 
 #ifdef	notdef
 /* AFS_KERBEROS_ENV is now conditionally defined in the Makefile */
@@ -132,7 +132,6 @@ int afs_tf_init(), afs_tf_get_pname(), afs_tf_get_pinst(), afs_tf_get_cred();
 int afs_tf_save_cred(), afs_tf_close(), afs_tf_create();
 int afs_tf_dest_tkt();
 static void ktc_LocalCell();
-char *ktc_tkt_string();
 #endif /* AFS_KERBEROS_ENV */
 
 #ifdef AFS_DUX40_ENV
@@ -1437,7 +1436,13 @@ char *getenv();
 static char krb_ticket_string[4096] = "";
 
 char *
-ktc_tkt_string()
+ktc_tkt_string(void)
+{
+    return ktc_tkt_string_uid(getuid());
+}
+
+char *
+ktc_tkt_string_uid(afs_uint32 uid)
 {
     char *env;
 
@@ -1450,7 +1455,7 @@ ktc_tkt_string()
 	} else {
 	    /* 32 bits of signed integer will always fit in 11 characters
 	     * (including the sign), so no need to worry about overflow */
-	    (void)sprintf(krb_ticket_string, "%s%d", TKT_ROOT, getuid());
+	    (void)sprintf(krb_ticket_string, "%s%d", TKT_ROOT, uid);
 	}
     }
     UNLOCK_GLOBAL_MUTEX;
@@ -1469,8 +1474,7 @@ ktc_tkt_string()
  */
 
 void
-ktc_set_tkt_string(val)
-     char *val;
+ktc_set_tkt_string(char * val)
 {
 
     LOCK_GLOBAL_MUTEX;
@@ -1608,6 +1612,13 @@ afs_tf_dest_tkt()
 static afs_uint32
 curpag()
 {
+#if defined(AFS_AIX51_ENV)
+    afs_int32 pag;
+
+    if (kcred_getpag(cred, PAG_AFS, &pag) < 0 || pag == 0)
+        pag = NOPAG;
+    return pag;
+#else
     gid_t groups[NGROUPS_MAX];
     afs_uint32 g0, g1;
     afs_uint32 h, l, ret;
@@ -1631,6 +1642,7 @@ curpag()
 	    return -1;
     }
     return -1;
+#endif
 }
 
 
