@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/ubik/remote.c,v 1.12.2.1 2004/08/25 07:09:43 shadow Exp $");
+    ("$Header: /cvs/openafs/src/ubik/remote.c,v 1.15.4.1 2006/12/15 16:39:46 shadow Exp $");
 
 #include <sys/types.h>
 #ifdef AFS_NT40_ENV
@@ -30,6 +30,7 @@ RCSID
 #include <lock.h>
 #include <rx/xdr.h>
 #include <rx/rx.h>
+#include <afs/afsutil.h>
 
 #define UBIK_INTERNALS
 #include "ubik.h"
@@ -455,9 +456,10 @@ SDISK_GetFile(rxcall, file, version)
     }
     length = ubikstat.size;
     tlen = htonl(length);
-    code = rx_Write(rxcall, &tlen, sizeof(afs_int32));
+    code = rx_Write(rxcall, (char *)&tlen, sizeof(afs_int32));
     if (code != sizeof(afs_int32)) {
 	DBRELE(dbase);
+	ubik_dprint("Rx-write length error=%d\n", code);
 	return BULK_ERROR;
     }
     offset = 0;
@@ -466,11 +468,13 @@ SDISK_GetFile(rxcall, file, version)
 	code = (*dbase->read) (dbase, file, tbuffer, offset, tlen);
 	if (code != tlen) {
 	    DBRELE(dbase);
+	    ubik_dprint("read failed error=%d\n", code);
 	    return UIOERROR;
 	}
 	code = rx_Write(rxcall, tbuffer, tlen);
 	if (code != tlen) {
 	    DBRELE(dbase);
+	    ubik_dprint("Rx-write length error=%d\n", code);
 	    return BULK_ERROR;
 	}
 	length -= tlen;
@@ -544,12 +548,14 @@ SDISK_SendFile(rxcall, file, length, avers)
 	code = rx_Read(rxcall, tbuffer, tlen);
 	if (code != tlen) {
 	    DBRELE(dbase);
+	    ubik_dprint("Rx-read length error=%d\n", code);
 	    code = BULK_ERROR;
 	    goto failed;
 	}
 	code = (*dbase->write) (dbase, file, tbuffer, offset, tlen);
 	if (code != tlen) {
 	    DBRELE(dbase);
+	    ubik_dprint("write failed error=%d\n", code);
 	    code = UIOERROR;
 	    goto failed;
 	}
